from uuid import uuid4

from fastfood_two.application.abstractions.interactor import Interactor
from fastfood_two.application.contracts.requests import AddMenuDTO
from fastfood_two.application.contracts.responses import MenuDTO
from fastfood_two.domain.menu.gateway import MenuGateway
from fastfood_two.domain.menu.menu_entity import Description, Menu, MenuId, Title


class AddMenu(Interactor[AddMenuDTO, MenuDTO]):
    def __init__(self, gateway: MenuGateway) -> None:
        self._menu_gateway = gateway

    async def __call__(self, request: AddMenuDTO) -> MenuDTO:

        menu = await self._menu_gateway.insert_menu(
            menu=Menu(
                id=MenuId(uuid4()) if request.id is None else MenuId(request.id),
                title=Title(request.title),
                description=Description(request.description),
            )
        )
        return MenuDTO(
            id=menu.id.value,
            title=menu.title.value,
            description=menu.description.value,
        )
