from fastfood_two.application.abstractions.interactor import Interactor
from fastfood_two.application.contracts.requests import GetMenuByIdDTO
from fastfood_two.application.contracts.responses import MenuDTO
from fastfood_two.domain.menu.error import MenuNotFoundError
from fastfood_two.domain.menu.gateway import MenuGateway
from fastfood_two.domain.menu.menu_entity import MenuId


class GetMenuById(Interactor[GetMenuByIdDTO, MenuDTO]):
    def __init__(self, gateway: MenuGateway) -> None:
        self._menu_gateway = gateway

    async def __call__(self, request: GetMenuByIdDTO) -> MenuDTO:

        menu = await self._menu_gateway.get_menu_by_id(
            id=MenuId(request.id),
        )

        if not menu:
            raise MenuNotFoundError(message="Menu not found")

        return MenuDTO(
            id=menu.id.value,
            title=menu.title.value,
            description=menu.description.value,
        )
