from fastfood_two.application.abstractions.interactor import Interactor
from fastfood_two.application.contracts.requests import UpdateMenuDTO
from fastfood_two.application.contracts.responses import MenuDTO
from fastfood_two.domain.menu.gateway import MenuGateway
from fastfood_two.domain.menu.menu_entity import Description, MenuId, Title


class UpdateMenu(Interactor[UpdateMenuDTO, MenuDTO]):
    def __init__(self, gateway: MenuGateway) -> None:
        self._menu_gateway = gateway

    async def __call__(self, request: UpdateMenuDTO) -> MenuDTO:

        menu = await self._menu_gateway.update_menu(
            id=MenuId(request.id),
            title=Title(request.title) if request.title else None,
            description=Description(request.description) if request.description else None,
        )

        return MenuDTO(
            id=menu.id.value,
            title=menu.title.value,
            description=menu.description.value,
        )
