from uuid import uuid4

from fastfood_two.application.abstractions.interactor import Interactor
from fastfood_two.application.contracts.requests import AddMenuDTO, AddSubMenuDTO
from fastfood_two.application.contracts.responses import SubMenuDTO
from fastfood_two.domain.submenu.gateway import SubMenuGateway
from fastfood_two.domain.submenu.submenu_entity import (
    Description,
    SubMenu,
    SubMenuId,
    Title,
)


class AddMenu(Interactor[AddSubMenuDTO, SubMenuDTO]):
    def __init__(self, gateway: SubMenuGateway) -> None:
        self._menu_gateway = gateway

    async def __call__(self, request: AddSubMenuDTO) -> SubMenuDTO:

        menu = await self._menu_gateway.insert_submenu(
            menu=SubMenu(
                id=SubMenuId(uuid4()) if request.id is None else SubMenuId(request.id),
                title=Title(request.title),
                description=Description(request.description),
            )
        )
        return SubMenuDTO(
            id=menu.id.value,
            parent_menu=menu.parent_menu.value,
            title=menu.title.value,
            description=menu.description.value,
        )
