from dataclasses import dataclass
from uuid import UUID

from fastfood_two.domain.core.entity import DomainEntity
from fastfood_two.domain.core.value_obj import DomainValueObject


@dataclass(frozen=True)
class MenuId(DomainValueObject):
    value: UUID


@dataclass(frozen=True)
class Title(DomainValueObject):
    value: str


@dataclass(frozen=True)
class Description(DomainValueObject):
    value: str | None


@dataclass
class Menu(DomainEntity[MenuId]):
    title: Title
    description: Description

    @staticmethod
    def create(id: UUID, title: str, description: str | None) -> "Menu":
        return Menu(
            id=MenuId(id),
            title=Title(title),
            description=Description(description),
        )
