from sqlalchemy import text
from sqlalchemy.ext.asyncio import AsyncSession

from fastfood_two.domain.menu.menu_entity import Description, Menu, MenuId, Title
from fastfood_two.infrastructure.pg_storage.mappers.menu_mapper import (
    db_entity_to_domain,
)


class MenuGatewayImpl:
    def __init__(self, session: AsyncSession) -> None:
        self._session = session

    async def get_all_menus(self) -> list[Menu]:
        query = text("SELECT * FROM menus;")
        menus = await self._session.execute(query)
        return [db_entity_to_domain(menu._tuple()) for menu in menus]

    async def get_menu_by_id(self, id: MenuId) -> Menu | None:
        query = text("SELECT * FROM menus WHERE id = :id;")
        menu = (await self._session.execute(query, {"id": id.value})).tuples().first()

        return db_entity_to_domain(menu) if menu else None

    async def insert_menu(self, menu: Menu) -> Menu:
        query = text("INSERT INTO menus (id, title, description) VALUES (:id, :title, :description);")
        await self._session.execute(
            query,
            {
                "id": menu.id.value,
                "title": menu.title.value,
                "description": menu.description.value,
            },
        )
        await self._session.commit()
        return menu

    async def update_menu(self, id: MenuId, title: Title | None, description: Description | None) -> Menu:
        query = text(
            "UPDATE menus SET {}{} {} WHERE id = :id RETURNING *;".format(
                "title = :title " if title is not None else "",
                "," if all([title is not None, description is not None]) else "",
                "description = :description" if description is not None else "",
            )
        )
        menu = await self._session.execute(
            query,
            {
                "id": id.value,
                "title": title.value if title else None,
                "description": description.value if description else None,
            },
        )
        await self._session.commit()
        return db_entity_to_domain(menu.one()._tuple())

    async def delete_menu(self, id: MenuId) -> None:
        query = text("DELETE FROM menus WHERE id = :id;")
        await self._session.execute(query, {"id": id.value})
        await self._session.commit()
