from sqlalchemy import ForeignKey
from sqlalchemy.orm import Mapped, mapped_column, relationship

from .base import Base
from .common_attrs import str_25, uuidpk


class SQLAMenu(Base):
    __tablename__ = "menus"

    id: Mapped[uuidpk] = mapped_column(primary_key=True, unique=True)
    title: Mapped[str_25]
    description: Mapped[str | None]

    parent: Mapped["SQLAMenu"] = mapped_column(ForeignKey("menus.id", ondelete="CASCADE"), nullable=True)

    submenus: Mapped[list["SQLAMenu"]] = relationship(
        "SQLAMenu", backref="menus", lazy="selectin", cascade="all, delete", remote_side=id
    )

    def to_json(self):
        return {
            "id": str(self.id),
            "title": self.title,
            "description": self.description,
        }
