import logging

from fastapi import FastAPI
from sqlalchemy.ext.asyncio import AsyncEngine, AsyncSession, async_sessionmaker

from fastfood_two.application.common.logger import configure_logger
from fastfood_two.application.config import Config
from fastfood_two.application.usecases.menu.add_menu import AddMenu
from fastfood_two.application.usecases.menu.delete_menu import DeleteMenu
from fastfood_two.application.usecases.menu.get_all_menus import GetAllMenus
from fastfood_two.application.usecases.menu.get_menu_by_id import GetMenuById
from fastfood_two.application.usecases.menu.update_menu import UpdateMenu
from fastfood_two.domain.menu.gateway import MenuGateway
from fastfood_two.presentation.fastapi_backend.depends.config import get_settings
from fastfood_two.presentation.fastapi_backend.depends.gateways import get_menu_gateway
from fastfood_two.presentation.fastapi_backend.depends.session import (
    create_engine,
    create_session_maker,
    get_session,
)
from fastfood_two.presentation.fastapi_backend.depends.usecases import (
    add_menu_usecase,
    delete_menu_usecase,
    get_all_menus_usecase,
    get_menu_by_id_usecase,
    update_menu_usecase,
)

logger = logging.getLogger(__name__)
configure_logger(level=logging.INFO)


def init_dependencies(app: FastAPI) -> None:
    """Initialize FastAPI dependencies.

    :param app: FastAPI application
    :type app: FastAPI
    """

    app.dependency_overrides[Config] = get_settings

    app.dependency_overrides[AsyncEngine] = create_engine
    app.dependency_overrides[async_sessionmaker[AsyncSession]] = create_session_maker
    app.dependency_overrides[AsyncSession] = get_session

    app.dependency_overrides[MenuGateway] = get_menu_gateway

    app.dependency_overrides[GetAllMenus] = get_all_menus_usecase
    app.dependency_overrides[AddMenu] = add_menu_usecase
    app.dependency_overrides[UpdateMenu] = update_menu_usecase
    app.dependency_overrides[DeleteMenu] = delete_menu_usecase
    app.dependency_overrides[GetMenuById] = get_menu_by_id_usecase

    logger.info("Dependencies initialized")
